// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

/// A utility class that provides a way to create a [BlocProvider] or
/// [RepositoryProvider] with a bloc or Repository that is already
/// available in the widget tree.
abstract class SmartProvider {
  /// Creates a [BlocProvider] with a bloc [B] that is possibly already
  /// available in the widget tree.
  static BlocProvider<B> bloc<B extends BlocBase<S>, S extends Object>(
    BuildContext context, {
    required B Function(BuildContext) create,
    required B Function(BuildContext, B) init,
    Widget? child,
    bool lazy = true,
    bool enable = true,
  }) {
    if (enable) {
      final bloc = context.read<B?>();
      if (bloc != null) {
        final b = bloc;
        return BlocProvider<B>.value(
          value: init(context, b),
          child: child,
        );
      }
    }

    return BlocProvider<B>(
      lazy: lazy,
      create: (_) => init(context, create(context)),
      child: child,
    );
  }

  /// Creates a [RepositoryProvider] with a Repository that is possibly
  /// already available in the widget tree.
  static RepositoryProvider<Repository> repo<Repository>(
    BuildContext context, {
    required Repository Function(BuildContext) create,
    required Repository Function(BuildContext, Repository) init,
    Widget? child,
    bool lazy = true,
    bool enable = true,
  }) {
    if (enable) {
      final repo = context.read<Repository?>();
      if (repo != null) {
        final r = repo;
        return RepositoryProvider<Repository>.value(
          value: init(context, r),
          child: child,
        );
      }
    }

    return RepositoryProvider<Repository>(
      lazy: lazy,
      create: (_) => init(context, create(context)),
      child: child,
    );
  }
}
