// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:io';

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_continuous_deployment/src/core/constants/app_constants.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/core/utils/logging.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/config/android_config.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/config/init_config_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/config/ios_config.dart';
import 'package:yaml_edit/yaml_edit.dart';

@usecase
class InitConfigUsecase extends AsyncUseCase<InitConfigParameters, void> {
  const InitConfigUsecase() : super();

  @override
  FutureOr<Res<void>> execute(InitConfigParameters params) => unsafe(() async {
        final file = File(params.path);
        if (!file.existsSync()) {
          logger.detail('Creating config file at `${params.path}`');
          file.createSync();
        } else {
          if (!params.force) {
            throw ClientException(
              'Config file already exists at `${params.path}`',
            );
          } else {
            logger.detail('Forced to create config file at `${params.path}`');
            file
              ..deleteSync()
              ..createSync();
          }
        }

        const androidConfig = AndroidConfig.defaultValues;

        logger.detail(
          'Writing default android config: ${androidConfig.toJson()}',
        );

        const iosConfig = IosConfig.defaultValues;

        logger.detail(
          'Writing default ios config: ${iosConfig.toJson()}',
        );

        final yamlEditor = YamlEditor('')
          ..update([], {
            AppConstants.androidConfigKey: androidConfig.toJson(),
            AppConstants.iosConfigKey: iosConfig.toJson(),
          });

        file.writeAsStringSync(yamlEditor.toString());
      });
}
